package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.List;
import java.util.Objects;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class ScoreComponentImpl extends AbstractComponent implements ScoreComponent {
   private final String name;
   private final String objective;
   /** @deprecated */
   @Deprecated
   private final @Nullable String value;

   static ScoreComponent create(final @NotNull List<? extends ComponentLike> children, final @NotNull Style style, final @NotNull String name, final @NotNull String objective, final @Nullable String value) {
      return new ScoreComponentImpl(ComponentLike.asComponents(children, IS_NOT_EMPTY), (Style)Objects.requireNonNull(style, "style"), (String)Objects.requireNonNull(name, "name"), (String)Objects.requireNonNull(objective, "objective"), value);
   }

   ScoreComponentImpl(final @NotNull List<Component> children, final @NotNull Style style, final @NotNull String name, final @NotNull String objective, final @Nullable String value) {
      super(children, style);
      this.name = name;
      this.objective = objective;
      this.value = value;
   }

   public @NotNull String name() {
      return this.name;
   }

   public @NotNull ScoreComponent name(final @NotNull String name) {
      return (ScoreComponent)(Objects.equals(this.name, name) ? this : create(this.children, this.style, name, this.objective, this.value));
   }

   public @NotNull String objective() {
      return this.objective;
   }

   public @NotNull ScoreComponent objective(final @NotNull String objective) {
      return (ScoreComponent)(Objects.equals(this.objective, objective) ? this : create(this.children, this.style, this.name, objective, this.value));
   }

   /** @deprecated */
   @Deprecated
   public @Nullable String value() {
      return this.value;
   }

   /** @deprecated */
   @Deprecated
   public @NotNull ScoreComponent value(final @Nullable String value) {
      return (ScoreComponent)(Objects.equals(this.value, value) ? this : create(this.children, this.style, this.name, this.objective, value));
   }

   public @NotNull ScoreComponent children(final @NotNull List<? extends ComponentLike> children) {
      return create(children, this.style, this.name, this.objective, this.value);
   }

   public @NotNull ScoreComponent style(final @NotNull Style style) {
      return create(this.children, style, this.name, this.objective, this.value);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof ScoreComponent)) {
         return false;
      } else if (!super.equals(other)) {
         return false;
      } else {
         ScoreComponent that = (ScoreComponent)other;
         return Objects.equals(this.name, that.name()) && Objects.equals(this.objective, that.objective()) && Objects.equals(this.value, that.value());
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      result = 31 * result + this.name.hashCode();
      result = 31 * result + this.objective.hashCode();
      result = 31 * result + Objects.hashCode(this.value);
      return result;
   }

   public String toString() {
      return Internals.toString(this);
   }

   @NotNull
   public ScoreComponent.@NotNull Builder toBuilder() {
      return new BuilderImpl(this);
   }

   static final class BuilderImpl extends AbstractComponentBuilder<ScoreComponent, ScoreComponent.Builder> implements ScoreComponent.Builder {
      private @Nullable String name;
      private @Nullable String objective;
      private @Nullable String value;

      BuilderImpl() {
      }

      BuilderImpl(final @NotNull ScoreComponent component) {
         super(component);
         this.name = component.name();
         this.objective = component.objective();
         this.value = component.value();
      }

      @NotNull
      public ScoreComponent.@NotNull Builder name(final @NotNull String name) {
         this.name = (String)Objects.requireNonNull(name, "name");
         return this;
      }

      @NotNull
      public ScoreComponent.@NotNull Builder objective(final @NotNull String objective) {
         this.objective = (String)Objects.requireNonNull(objective, "objective");
         return this;
      }

      /** @deprecated */
      @Deprecated
      @NotNull
      public ScoreComponent.@NotNull Builder value(final @Nullable String value) {
         this.value = value;
         return this;
      }

      public @NotNull ScoreComponent build() {
         if (this.name == null) {
            throw new IllegalStateException("name must be set");
         } else if (this.objective == null) {
            throw new IllegalStateException("objective must be set");
         } else {
            return ScoreComponentImpl.create(this.children, this.buildStyle(), this.name, this.objective, this.value);
         }
      }
   }
}
